/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_app.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <time.h>
#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_app.h"
#include "tv_menu.h"
#include "tv_view.h"
#include "tv_registry.h"
#include "tv_event.h"
/* }}} */

extern TV_CLASS_ENTRY(TMenuItem);
extern TV_CLASS_ENTRY(TGroup);
extern TV_CLASS_ENTRY(TDeskTop);
extern TV_CLASS_ENTRY(TEvent);

TApp::TApp(zval *_thisptr) : thisptr(_thisptr) ,/*{{{*/
    TProgInit( &TApplication::initStatusLine,
               &TApplication::initMenuBar,
               &TApplication::initDeskTop
             )
{
	zval method_name, *retval;
	TRect r;
	TV_OBJ(TMenuItem)* mnu;
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "initMenu", strlen("initMenu"), 1);
	if (call_user_function_ex(NULL, &thisptr, &method_name, &retval, 0, 0, 1, NULL TSRMLS_CC) == SUCCESS) {
		if (!ZVAL_IS_NULL(retval) && (Z_TYPE_P(retval) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(retval), TV_CE(TMenuItem) TSRMLS_CC))) {
			php_error(E_WARNING, "Menu initialization function must return instance of tvTMenu or NULL");
		} else {
			if (!ZVAL_IS_NULL(retval)) {
				mnu=(TV_OBJ(TMenuItem)*)zend_object_store_get_object(retval);
				r = getExtent();
				r.b.y = r.a.y + 1;    // set bottom line 1 line below top line
				menuBar = new TMenuBar(r, new TMenu(*mnu->obj));
				insert(menuBar);
			}
		}
	}
}/*}}}*/

void TApp::idle()/*{{{*/
{
	zval method_name, *retval;
	TApplication::idle();
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "idle", strlen("idle"), 1);
	call_user_function_ex(NULL, &thisptr, &method_name, &retval, 0, 0, 1, NULL TSRMLS_CC);
}/*}}}*/


TV_STD_HANDLEEVENT_BEGIN(TApp)
TApplication::handleEvent(event);
TV_STD_HANDLEEVENT_BODY
TV_STD_HANDLEEVENT_END

TV_CLASS_ENTRY(TApplication);

TV_CE_BEGIN(TApplication)/*{{{*/
	PHP_ME(TApplication, __construct,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TApplication, run,				NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TApplication, idle,				NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TApplication, commandHandler,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TApplication, deskTop,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TApplication, stopTimer,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TApplication, startTimer,		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TApplication)/*}}}*/

TV_OBJECTS_NEW(TApplication)

TV_CE_INIT(TApplication, TV_CE(TGroup))

PHP_METHOD(TApplication, __construct) /* {{{ */
{
	char *callback_name;
	TV_OBJ(TApplication)* app;
	TApp *tvapp;
	app = (TV_OBJ(TApplication)*)zend_object_store_get_object(getThis());
	tvapp = new TApp(getThis());
	app->obj = tvapp;
	addref_and_store(app);
}
/* }}} */

PHP_METHOD(TApplication, run) /* {{{ */
{
	struct itimerval timer;
	TV_OBJ(TApplication)* app;
	app=(TV_OBJ(TApplication)*)zend_object_store_get_object(getThis());
	app->obj->run();
	memset(&timer, 0, sizeof(timer));
	setitimer (ITIMER_REAL, &timer, NULL);
}
/* }}} */

PHP_METHOD(TApplication, stopTimer) /* {{{ */
{
	struct itimerval timer;
	memset(&timer, 0, sizeof(timer));
	setitimer (ITIMER_REAL, &timer, NULL);
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TApplication, startTimer) /* {{{ */
{
	struct itimerval timer;
    timer.it_interval.tv_usec = timer.it_value.tv_usec = 100000;
    timer.it_interval.tv_sec = timer.it_value.tv_sec = 0;
	setitimer (ITIMER_REAL, &timer, NULL);
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TApplication, commandHandler) /* {{{ */
{
	RETURN_FALSE;
}
/* }}} */

PHP_METHOD(TApplication, idle) /* {{{ */
{
	RETURN_FALSE;
}
/* }}} */

PHP_METHOD(TApplication, deskTop) /* {{{ */
{
	TV_OBJ(TApplication)* app;
	TV_OBJ(TDeskTop)* newval;
	app=(TV_OBJ(TApplication)*)zend_object_store_get_object(getThis());
	object_init_ex(return_value, TV_CE(TDeskTop));
    newval=(TV_OBJ(TDeskTop)*)zend_object_store_get_object(return_value);
    newval->obj = app->obj->deskTop;
}
/* }}} */

/*
 *  * Local variables:
 *  * tab-width: 4
 *  * c-basic-offset: 4
 *  * End:
 *  * vim600: noet sw=4 ts=4 fdm=marker
 *  * vim<600: noet sw=4 ts=4
 *  
 */

