/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_common.h 2 2005-04-17 13:24:23Z andrey $ */

#define TV_ALIASED_OBJECT_BEGIN(classname, cppclassname) typedef struct _tv_object_##classname { \
    zend_object zo; \
    cppclassname *obj; \
    char ref;
#define TV_OBJECT_END(classname) } tv_object_##classname;

#define TV_OBJECT_BEGIN(classname) TV_ALIASED_OBJECT_BEGIN(classname, classname)

#define TV_OBJECT(classname) TV_OBJECT_BEGIN(classname) TV_OBJECT_END(classname)
#define TV_ALIASED_OBJECT(classname, cppclassname) TV_ALIASED_OBJECT_BEGIN(classname, cppclassname) TV_OBJECT_END(classname)

#define TV_OBJ(classname) tv_object_##classname

#define TV_CLASS_ENTRY(classname) zend_class_entry *tv_##classname##_class_entry
#define TV_CE(classname) tv_##classname##_class_entry

#define TV_FETCH_OBJECT(object, classname)  ((tv_object_##classname *)zend_object_store_get_object(object))->obj

#define TV_OBJECTS_NEW_N(classname) ZEND_API zend_object_value tv_objects_##classname##_new(zend_class_entry *class_type TSRMLS_DC)

#define TV_OBJECTS_NEW(classname) TV_OBJECTS_NEW_N(classname) \
{\
    zend_object_value retval; \
    TV_OBJ(classname) *intern; \
    zval *tmp; \
    intern = (TV_OBJ(classname) *)emalloc(sizeof(TV_OBJ(classname))); \
    intern->ref = 0; \
    intern->zo.ce = class_type; \
    intern->zo.in_get = 0; \
    intern->zo.in_set = 0; \
    ALLOC_HASHTABLE(intern->zo.properties); \
    zend_hash_init(intern->zo.properties, 0, NULL, ZVAL_PTR_DTOR, 0); \
    zend_hash_copy( \
        intern->zo.properties, &class_type->default_properties, \
        (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *) \
    ); \
	memcpy(&tv_##classname##_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers)); \
    retval.handle = zend_objects_store_put(intern, NULL, NULL, NULL TSRMLS_CC); \
    retval.handlers = zend_get_std_object_handlers(); /*&tv_##classname##_object_handlers;*/ \
    return retval; \
}

#define TV_INIT_CE(classname) tv_init_##classname##_ce()

#define TV_CE_INIT_D(classname) void tv_init_##classname##_ce()
#define TV_CE_INIT(classname, parent) void tv_init_##classname##_ce() \
{\
    zend_class_entry ce;\
    INIT_CLASS_ENTRY(ce, #classname, tv_##classname##_class_functions);\
	ce.create_object = tv_objects_##classname##_new; \
    TV_CE(classname) = zend_register_internal_class_ex(&ce, parent, NULL TSRMLS_CC);\
}

#define TV_CE_BEGIN(classname) static zend_function_entry tv_##classname##_class_functions[] = \
{

#define TV_CE_END(classname) {NULL, NULL, NULL} }; static zend_object_handlers tv_##classname##_object_handlers;

#define TV_LONG_PROP_GETTER_SETTER(classname, name, has_setter) PHP_METHOD(classname, name) \
{ \
	TV_OBJ(classname)* obj;\
	long newval;\
	if (has_setter) {\
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &newval) == FAILURE) {\
		RETURN_NULL();\
	} \
	} \
	obj=(TV_OBJ(classname)*)zend_object_store_get_object(getThis());\
	if (has_setter) {\
	if (ZEND_NUM_ARGS()) {\
		obj->obj->name = newval;\
	}\
	}\
	RETVAL_LONG((long)obj->obj->name);\
}

#define TV_BOOL_PROP_GETTER_SETTER(classname, name, has_setter) PHP_METHOD(classname, name) \
{ \
	TV_OBJ(classname)* obj;\
	zend_bool newval;\
	if (has_setter) {\
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &newval) == FAILURE) {\
		RETURN_NULL();\
	} \
	} \
	obj=(TV_OBJ(classname)*)zend_object_store_get_object(getThis());\
	if (has_setter) {\
	if (ZEND_NUM_ARGS()) {\
		obj->obj->name = newval ? True : False;\
	}\
	}\
	if (obj->obj->name) {\
		RETURN_TRUE;\
	} else {\
		RETURN_FALSE;\
	}\
}


#define TV_STRING_PROP_GETTER_SETTER(classname, name, has_setter) PHP_METHOD(classname, name) \
{ \
	TV_OBJ(classname)* obj;\
	char *newval;\
	int vallen; \
	if (has_setter) {\
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &newval, &vallen) == FAILURE) {\
		RETURN_NULL();\
	} \
	} \
	obj=(TV_OBJ(classname)*)zend_object_store_get_object(getThis());\
	if (has_setter) {\
	if (ZEND_NUM_ARGS()) {\
		obj->obj->name = newval;\
	}\
	}\
	RETVAL_STRING((char*)obj->obj->name, 1);\
}


#define TV_OBJ_PROP_GETTER_SETTER(classname, retclassname, name, has_setter) PHP_METHOD(classname, name) \
{ \
	TV_OBJ(classname)* obj;\
	TV_OBJ(retclassname)* newval;\
	zval *newzval;\
	if (has_setter) {\
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|O!", &newzval, TV_CE(retclassname)) == FAILURE) {\
		RETURN_NULL();\
	} \
	} \
	obj=(TV_OBJ(classname)*)zend_object_store_get_object(getThis());\
	if (has_setter) {\
	if (ZEND_NUM_ARGS()) {\
		if (ZVAL_IS_NULL(newzval)) {\
			obj->obj->name = NULL; \
		} else {\
			newval=(TV_OBJ(retclassname)*)zend_object_store_get_object(newzval);\
			obj->obj->name = newval->obj;\
		}\
	}\
	}\
	if (obj->obj->name) {\
		object_init_ex(return_value, TV_CE(retclassname));\
		newval=(TV_OBJ(retclassname)*)zend_object_store_get_object(return_value);\
		newval->obj = obj->obj->name;\
	} else {\
		RETURN_NULL();\
	}\
}

#define ALLOC_CALLBACK_ARGS(a, b, c)\
if (c) {\
    a = (zval ***)safe_emalloc(c, sizeof(zval **), 0);\
    for (int i = b; i < c; i++) {\
        a[i] = (zval **)emalloc(sizeof(zval *));\
        MAKE_STD_ZVAL(*a[i]);\
    }\
}

#define FREE_CALLBACK_ARGS(a, b, c)\
if (a) {\
    for (int i=b; i < c; i++) {\
        zval_ptr_dtor(a[i]);\
        efree(a[i]);\
    }\
    efree(a);\
}


#define addref_once(zobj) \
    if (!zobj->ref) { \
        zobj->ref = 1; \
        ZVAL_ADDREF(getThis());\
    }

#define addref_and_store(zobj) \
    addref_once(zobj) \
    tv_registry_put(zobj->obj, getThis())




#define TV_STD_HANDLEEVENT_BEGIN(klass) void klass::handleEvent(TEvent& event) \
{ \
	zval method_name, *retval; \
	zval ***call_args; \




#define TV_STD_HANDLEEVENT_BODY \
	if (event.what) { \
		zval *zevent; \
		zval *thisptr = tv_registry_get(this); \
		zval *tmp, *tmp1; \
		TV_OBJ(TEvent) *wevent; \
		MAKE_STD_ZVAL(zevent); \
		object_init_ex(zevent, TV_CE(TEvent)); \
		wevent=(TV_OBJ(TEvent)*)zend_object_store_get_object(zevent); \
		wevent->obj = new TEvent(event); \
		INIT_PZVAL(&method_name); \
		ZVAL_STRINGL(&method_name, "handleEvent", sizeof("handleEvent")-1, 0); \
		ALLOC_CALLBACK_ARGS(call_args, 0, 1); \
		*call_args[0] = zevent; \
		if (call_user_function_ex(NULL, &thisptr, &method_name, &retval, 1, call_args, 0, NULL TSRMLS_CC) != FAILURE) { \
			/* */ \
		} \
		FREE_CALLBACK_ARGS(call_args, 0, 1) \
	} \


#define TV_STD_HANDLEEVENT_END }

