/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_dialog.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_dialog.h"
#include "tv_view.h"
#include "tv_registry.h"
#include "tv_event.h"
/* }}} */

extern TV_CLASS_ENTRY(TWindow);
extern TV_CLASS_ENTRY(TView);
extern TV_CLASS_ENTRY(TEvent);

TVDialog::TVDialog(const TRect& bounds, const char *title) :/*{{{*/
	TDialog(bounds, title), TWindowInit( &TVDialog::initFrame )
{
}/*}}}*/

TVDialog::~TVDialog()/*{{{*/
{
}/*}}}*/

TV_STD_HANDLEEVENT_BEGIN(TVDialog)/*{{{*/
TDialog::handleEvent(event);
TV_STD_HANDLEEVENT_BODY
TV_STD_HANDLEEVENT_END/*}}}*/


TVCheckBoxes::TVCheckBoxes(const TRect& bounds, TSItem *list) :/*{{{*/
	TCheckBoxes(bounds, list) 
{
}/*}}}*/

void TVCheckBoxes::press(int item)/*{{{*/
{
	TCheckBoxes::press(item);
	message(owner, evBroadcast, cmClusterChanged, this);
}/*}}}*/

TVCheckBoxes::~TVCheckBoxes()/*{{{*/
{
}/*}}}*/


TVRadioButtons::TVRadioButtons(const TRect& bounds, TSItem *list) :/*{{{*/
	TRadioButtons(bounds, list) 
{
}/*}}}*/

void TVRadioButtons::movedTo(int item)/*{{{*/
{
	TRadioButtons::movedTo(item);
	message(owner, evBroadcast, cmClusterChanged, this);
}/*}}}*/

void TVRadioButtons::press(int item)/*{{{*/
{
	TRadioButtons::press(item);
	message(owner, evBroadcast, cmClusterChanged, this);
}/*}}}*/

TVRadioButtons::~TVRadioButtons()/*{{{*/
{
}/*}}}*/


TProgressBar::TProgressBar(const TRect &r, int aMax, int aValue): TView(r),/*{{{*/
	max(aMax), value(aValue)
{
	growMode |= gfGrowHiX;
}/*}}}*/

void TProgressBar::draw()/*{{{*/
{
	TDrawBuffer b;
	char str[maxViewWidth];
	int mid = (max == 0 || value >= max) ? size.x : size.x * value / max;

	sprintf(str, "%d%%", max == 0 ? 100 : 100 * value / max);
	b.moveChar(0, ' ', colLeft, mid);
	b.moveChar(mid, ' ', colRight, size.x - mid);
	b.moveStr((size.x - strlen(str)) / 2, str, 0);
	writeBuf(0, 0, size.x, size.y, b);
}/*}}}*/

void TProgressBar::setValue(int aValue)/*{{{*/
{
	if (aValue != value)
	{
		value = aValue;
		drawView();
	}
}/*}}}*/


TVListBox::TVListBox(const TRect& bounds, ushort aNumCols, TScrollBar *aScrollBar ) :/*{{{*/
	TListBox(bounds, aNumCols, aScrollBar )
{
}/*}}}*/

void TVListBox::focusItemNum(short item)/*{{{*/
{
	TListBox::focusItemNum(item);
	selectItem(item);
}/*}}}*/


TVStaticText::TVStaticText(const TRect& bounds, const char *aText) :/*{{{*/
	TStaticText(bounds, aText)
{

}/*}}}*/

void TVStaticText::setText(const char *aText)/*{{{*/
{
	delete(char*) text;
	(char*) text = newStr(aText);
	drawView();
}/*}}}*/


/* TDialog declarations {{{ */
TV_CLASS_ENTRY(TDialog);
TV_CE_BEGIN(TDialog)
	PHP_ME(TDialog, __construct, NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TDialog)
TV_OBJECTS_NEW(TDialog)
TV_CE_INIT(TDialog, TV_CE(TWindow))
/* }}} */

/* TInputLine declarations {{{ */
TV_CLASS_ENTRY(TInputLine);
TV_CE_BEGIN(TInputLine)
	PHP_ME(TInputLine, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TInputLine, selectAll,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TInputLine, selStart,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TInputLine, selEnd,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TInputLine, getData,		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TInputLine)
TV_OBJECTS_NEW(TInputLine)
TV_CE_INIT(TInputLine, TV_CE(TView))
/* }}} */

/* TButton declarations {{{ */
TV_CLASS_ENTRY(TButton);
TV_CE_BEGIN(TButton)
	PHP_ME(TButton, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TButton, press,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TButton, makeDefault,	NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TButton)
TV_OBJECTS_NEW(TButton)
TV_CE_INIT(TButton, TV_CE(TView))
/* }}} */

/* TCheckBoxes declarations {{{ */
TV_CLASS_ENTRY(TCheckBoxes);
TV_CE_BEGIN(TCheckBoxes)
	PHP_ME(TCheckBoxes, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TCheckBoxes, checked,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TCheckBoxes, toggle,			NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TCheckBoxes)
TV_OBJECTS_NEW(TCheckBoxes)
TV_CE_INIT(TCheckBoxes, TV_CE(TView))
/* }}} */

/* TRadioButtons declarations {{{ */
TV_CLASS_ENTRY(TRadioButtons);
TV_CE_BEGIN(TRadioButtons)
	PHP_ME(TRadioButtons, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRadioButtons, checked,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRadioButtons, check,		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TRadioButtons)
TV_OBJECTS_NEW(TRadioButtons)
TV_CE_INIT(TRadioButtons, TV_CE(TView))
/* }}} */

/* TScrollBar declarations {{{ */
TV_CLASS_ENTRY(TScrollBar);
TV_CE_BEGIN(TScrollBar)
	PHP_ME(TScrollBar, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TScrollBar, setParams,   NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TScrollBar, setRange,    NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TScrollBar, setStep,	    NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TScrollBar, setValue,    NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TScrollBar)
TV_OBJECTS_NEW(TScrollBar)
TV_CE_INIT(TScrollBar, TV_CE(TView))
/* }}} */

/* TListBox declarations {{{ */
TV_CLASS_ENTRY(TListBox);
TV_CE_BEGIN(TListBox)
	PHP_ME(TListBox, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TListBox, newList,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TListBox, focusedItem,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TListBox, selectItem,	NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TListBox)
TV_OBJECTS_NEW(TListBox)
TV_CE_INIT(TListBox, TV_CE(TView))
/* }}} */

/* TStaticText declarations {{{ */
TV_CLASS_ENTRY(TStaticText);
TV_CE_BEGIN(TStaticText)
	PHP_ME(TStaticText, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TStaticText, setText,		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TStaticText)
TV_OBJECTS_NEW(TStaticText)
TV_CE_INIT(TStaticText, TV_CE(TView))
/* }}} */

/* TLabel declarations {{{ */
TV_CLASS_ENTRY(TLabel);
TV_CE_BEGIN(TLabel)
	PHP_ME(TLabel, __construct,	NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TLabel)
TV_OBJECTS_NEW(TLabel)
TV_CE_INIT(TLabel, TV_CE(TStaticText))
/* }}} */

/* THistory declarations {{{ */
TV_CLASS_ENTRY(THistory);
TV_CE_BEGIN(THistory)
	PHP_ME(THistory, __construct,	NULL, ZEND_ACC_PUBLIC)
TV_CE_END(THistory)
TV_OBJECTS_NEW(THistory)
TV_CE_INIT(THistory, TV_CE(TView))
/* }}} */

/* TProgressBar declarations {{{ */
TV_CLASS_ENTRY(TProgressBar);
TV_CE_BEGIN(TProgressBar)
	PHP_ME(TProgressBar, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TProgressBar, getValue,	    NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TProgressBar, setValue,	    NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TProgressBar)
TV_OBJECTS_NEW(TProgressBar)
TV_CE_INIT(TProgressBar, TV_CE(TView))
/* }}} */

PHP_METHOD(TDialog, __construct) /* {{{ */
{
	TV_OBJ(TDialog) *obj;
	long ax, ay, bx, by;
	char *title;
	int title_len;

	title = ""; ax = ay = 1; bx = by = 8; // just some defaults

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllls", &ax, &ay, &bx, &by, &title, &title_len);
	obj = (TV_OBJ(TDialog)*)zend_object_store_get_object(getThis());
	obj->obj = new TVDialog(TRect(ax, ay, bx, by), title);
	addref_and_store(obj);

}
/* }}} */

PHP_METHOD(TInputLine, __construct) /* {{{ */
{
	TV_OBJ(TInputLine) *obj;
	long ax, ay, bx, by, max_len;

	ax = ay = 1; bx = 8; by = 2; // just some defaults

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllll", &ax, &ay, &bx, &by, &max_len);
	obj = (TV_OBJ(TInputLine)*)zend_object_store_get_object(getThis());
	obj->obj = new TInputLine(TRect(ax, ay, bx, by), max_len);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TInputLine, selectAll) /* {{{ */
{
	TV_OBJ(TInputLine) *obj;
	zend_bool enable;

	enable = 1;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &enable);
	obj = (TV_OBJ(TInputLine)*)zend_object_store_get_object(getThis());
	obj->obj->selectAll(enable ? True : False);
}
/* }}} */

PHP_METHOD(TInputLine, selStart) /* {{{ */
{
	TV_OBJ(TInputLine) *obj;
	long ret;
	obj = (TV_OBJ(TInputLine)*)zend_object_store_get_object(getThis());
	ret = obj->obj->selStart;
	RETVAL_LONG(ret);
}
/* }}} */

PHP_METHOD(TInputLine, selEnd) /* {{{ */
{
	TV_OBJ(TInputLine) *obj;
	long ret;
	obj = (TV_OBJ(TInputLine)*)zend_object_store_get_object(getThis());
	ret = obj->obj->selEnd;
	RETVAL_LONG(ret);
}
/* }}} */

PHP_METHOD(TInputLine, getData) /* {{{ */
{
	TV_OBJ(TInputLine) *obj;
	long datasize;
	char *data;
	obj = (TV_OBJ(TInputLine)*)zend_object_store_get_object(getThis());
	datasize = obj->obj->dataSize();
	data = (char*)emalloc(datasize + 1);
	data[datasize] = 0;
	obj->obj->getData(data);
	RETVAL_STRING(data, 0);
}
/* }}} */

PHP_METHOD(TButton, __construct) /* {{{ */
{
	TV_OBJ(TButton) *obj;
	long ax, ay, bx, by, cmd, flags;
	char *title;
	int title_len;

	title = "?"; ax = ay = 1; bx = 6; by = 3; // just some defaults

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllsll", &ax, &ay, &bx, &by, &title, &title_len, &cmd, &flags);
	obj = (TV_OBJ(TButton)*)zend_object_store_get_object(getThis());
	obj->obj = new TButton(TRect(ax, ay, bx, by), title, cmd, flags);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TButton, press) /* {{{ */
{
	TV_OBJ(TButton) *obj;
	obj = (TV_OBJ(TButton)*)zend_object_store_get_object(getThis());
	obj->obj->press();
}
/* }}} */

PHP_METHOD(TButton, makeDefault) /* {{{ */
{
	TV_OBJ(TButton) *obj;
	zend_bool enable;

	enable = 1;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &enable);
	obj = (TV_OBJ(TButton)*)zend_object_store_get_object(getThis());
	obj->obj->makeDefault(enable ? True : False);
}
/* }}} */

PHP_METHOD(TCheckBoxes, __construct) /* {{{ */
{
	TV_OBJ(TCheckBoxes) *obj;
	TSItem *list, *current;
	long ax, ay, bx, by;
	zval *items, **wval, strval;
	HashPosition pos;

	ax = ay = 1; bx = 6; by = 3; // just some defaults
	list = NULL;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllla", &ax, &ay, &bx, &by, &items) != FAILURE) {
		zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(items), &pos);
		while (SUCCESS == zend_hash_get_current_data_ex(Z_ARRVAL_P(items), (void**)&wval, &pos)) {
			strval = **wval;
			if (Z_TYPE_P(*wval) != IS_STRING) {
				zval_copy_ctor(&strval);
				convert_to_string(&strval);
			}
			
			if (!list) {
				list = new TSItem(Z_STRVAL(strval), NULL);
				current = list;
			} else {
				current->next = new TSItem(Z_STRVAL(strval), NULL);
				current = current->next;
			}
			
			if (Z_TYPE_P(*wval) != IS_STRING) {
				zval_dtor(&strval);
			}
			zend_hash_move_forward_ex(Z_ARRVAL_P(items), &pos);
		}
	}

	obj = (TV_OBJ(TCheckBoxes)*)zend_object_store_get_object(getThis());
	obj->obj = new TVCheckBoxes(TRect(ax, ay, bx, by), list);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TCheckBoxes, checked) /* {{{ */
{
	TV_OBJ(TCheckBoxes) *obj;
	long num;

	num = 200;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &num);
	obj = (TV_OBJ(TCheckBoxes)*)zend_object_store_get_object(getThis());
	num = obj->obj->mark(num);
	RETVAL_LONG(num);
}
/* }}} */

PHP_METHOD(TCheckBoxes, toggle) /* {{{ */
{
	TV_OBJ(TCheckBoxes) *obj;
	long num;

	num = 200;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &num);
	obj = (TV_OBJ(TCheckBoxes)*)zend_object_store_get_object(getThis());
	obj->obj->press(num);
}
/* }}} */

PHP_METHOD(TRadioButtons, __construct) /* {{{ */
{
	TV_OBJ(TRadioButtons) *obj;
	TSItem *list, *current;
	long ax, ay, bx, by;
	zval *items, **wval, strval;
	HashPosition pos;

	ax = ay = 1; bx = 6; by = 3; // just some defaults
	list = NULL;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllla", &ax, &ay, &bx, &by, &items) != FAILURE) {
		zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(items), &pos);
		while (SUCCESS == zend_hash_get_current_data_ex(Z_ARRVAL_P(items), (void**)&wval, &pos)) {
			strval = **wval;
			if (Z_TYPE_P(*wval) != IS_STRING) {
				zval_copy_ctor(&strval);
				convert_to_string(&strval);
			}
			
			if (!list) {
				list = new TSItem(Z_STRVAL(strval), NULL);
				current = list;
			} else {
				current->next = new TSItem(Z_STRVAL(strval), NULL);
				current = current->next;
			}
			
			if (Z_TYPE_P(*wval) != IS_STRING) {
				zval_dtor(&strval);
			}
			zend_hash_move_forward_ex(Z_ARRVAL_P(items), &pos);
		}
	}

	obj = (TV_OBJ(TRadioButtons)*)zend_object_store_get_object(getThis());
	obj->obj = new TVRadioButtons(TRect(ax, ay, bx, by), list);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TRadioButtons, checked) /* {{{ */
{
	TV_OBJ(TRadioButtons) *obj;
	ushort ret;
	obj = (TV_OBJ(TRadioButtons)*)zend_object_store_get_object(getThis());
	obj->obj->getData(&ret);
	RETVAL_LONG(ret);
}
/* }}} */

PHP_METHOD(TRadioButtons, check) /* {{{ */
{
	TV_OBJ(TRadioButtons) *obj;
	long num;
	ushort snum;

	num = 200;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &num);
	snum = num;
	obj = (TV_OBJ(TRadioButtons)*)zend_object_store_get_object(getThis());
	obj->obj->setData(&snum);
	obj->obj->press(snum);
}
/* }}} */

PHP_METHOD(TScrollBar, __construct) /* {{{ */
{
	TV_OBJ(TScrollBar) *obj;
	long ax, ay, bx, by;

	ax = ay = 1; bx = 2; by = 6; // just some defaults
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llll", &ax, &ay, &bx, &by);
	obj = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(getThis());
	obj->obj = new TScrollBar(TRect(ax, ay, bx, by));
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TScrollBar, setParams) /* {{{ */
{
	TV_OBJ(TScrollBar) *obj;
	long value, min, max, pgstep, arstep;

	value = min = 0;
	max = 100;
	arstep = 1;
	pgstep = 10;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllll", &value, &min, &max, &pgstep, &arstep);
	obj = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(getThis());
	obj->obj->setParams(value, min, max, pgstep, arstep);
}
/* }}} */

PHP_METHOD(TScrollBar, setRange) /* {{{ */
{
	TV_OBJ(TScrollBar) *obj;
	long min, max;

	min = 0;
	max = 100;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &min, &max);
	obj = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(getThis());
	obj->obj->setRange(min, max);
}
/* }}} */

PHP_METHOD(TScrollBar, setStep) /* {{{ */
{
	TV_OBJ(TScrollBar) *obj;
	long pgstep, arstep;

	pgstep = 10;
	arstep = 1;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &pgstep, &arstep);
	obj = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(getThis());
	obj->obj->setStep(pgstep, arstep);
}
/* }}} */

PHP_METHOD(TScrollBar, setValue) /* {{{ */
{
	TV_OBJ(TScrollBar) *obj;
	long value;

	value = 0;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &value);
	obj = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(getThis());
	obj->obj->setValue(value);
}
/* }}} */

PHP_METHOD(TListBox, __construct) /* {{{ */
{
	TV_OBJ(TListBox) *obj;
	TV_OBJ(TScrollBar) *sb;
	zval *zsb;
	long ax, ay, bx, by, numcols;

	ax = ay = 1; bx = 10; by = 6; numcols = 1; sb = NULL; zsb = NULL;// just some defaults
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llll|lO!", &ax, &ay, &bx, &by, &numcols, &zsb, TV_CE(TScrollBar));

	if (zsb && ! ZVAL_IS_NULL(zsb)) {
		sb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zsb);
	}
	obj = (TV_OBJ(TListBox)*)zend_object_store_get_object(getThis());
	obj->obj = new TVListBox(TRect(ax, ay, bx, by), numcols, sb ? sb->obj : NULL);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TListBox, newList) /* {{{ */
{
	TV_OBJ(TListBox) *obj;
	zval *items, **wval, strval;
	HashPosition pos;
	TNotSortedStrCollection	 *list;

	list = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &items) == FAILURE) {
		RETURN_FALSE;
	}

	zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(items), &pos);
	while (SUCCESS == zend_hash_get_current_data_ex(Z_ARRVAL_P(items), (void**)&wval, &pos)) {
		strval = **wval;
		if (Z_TYPE_P(*wval) != IS_STRING) {
			zval_copy_ctor(&strval);
			convert_to_string(&strval);
		}
		
		if (!list) {
			list = new TNotSortedStrCollection(5, 10);
		}
		list->insert(Z_STRVAL(strval));
		
		if (Z_TYPE_P(*wval) != IS_STRING) {
			zval_dtor(&strval);
		}
		zend_hash_move_forward_ex(Z_ARRVAL_P(items), &pos);
	}

	obj = (TV_OBJ(TListBox)*)zend_object_store_get_object(getThis());
	obj->obj->newList(list);
}
/* }}} */

PHP_METHOD(TListBox, selectItem) /* {{{ */
{
	TV_OBJ(TListBox) *obj;
	long item;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &item) == FAILURE) {
		RETURN_FALSE;
	}
	obj = (TV_OBJ(TListBox)*)zend_object_store_get_object(getThis());
	obj->obj->focusItemNum(item);
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TListBox, focusedItem) /* {{{ */
{
	TV_OBJ(TListBox) *obj;
	long item;

	obj = (TV_OBJ(TListBox)*)zend_object_store_get_object(getThis());
	item = obj->obj->focused;
	RETVAL_LONG(item);
}
/* }}} */

PHP_METHOD(TStaticText, __construct) /* {{{ */
{
	TV_OBJ(TStaticText) *obj;
	char *text;
	long ax, ay, bx, by;
	int text_len;

	ax = ay = 1; bx = 10; by = 2; text = ""; // just some defaults
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllls", &ax, &ay, &bx, &by, &text, &text_len);
	obj = (TV_OBJ(TStaticText)*)zend_object_store_get_object(getThis());
	obj->obj = new TVStaticText(TRect(ax, ay, bx, by), text);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TStaticText, setText) /* {{{ */
{
	TV_OBJ(TStaticText) *obj;
	char *text;
	int text_len;

	text = ""; // just some defaults
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &text, &text_len);
	obj = (TV_OBJ(TStaticText)*)zend_object_store_get_object(getThis());
	obj->obj->setText(text);
}
/* }}} */

PHP_METHOD(TLabel, __construct) /* {{{ */
{
	TV_OBJ(TLabel) *obj;
	TV_OBJ(TView)  *link;
	char *text;
	long ax, ay, bx, by;
	int text_len;
	zval *zlink;


	ax = ay = 1; bx = 10; by = 2; text = ""; // just some defaults
	zlink = NULL;
	link = NULL;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllsO", &ax, &ay, &bx, &by, &text, &text_len, &zlink, TV_CE(TView));
	if (zlink) {
		link = (TV_OBJ(TView)*)zend_object_store_get_object(zlink);
	}
	obj = (TV_OBJ(TLabel)*)zend_object_store_get_object(getThis());
	obj->obj = new TLabel(TRect(ax, ay, bx, by), text, link ? link->obj : NULL);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(THistory, __construct) /* {{{ */
{
	TV_OBJ(THistory) *obj;
	TV_OBJ(TInputLine)  *link;
	long ax, ay, bx, by, histid;
	zval *zlink;


	ax = ay = 1; bx = by = 2;  // just some defaults
	zlink = NULL;
	link = NULL;
	histid = 0;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllOl", &ax, &ay, &bx, &by, &zlink, TV_CE(TInputLine), &histid);
	if (zlink) {
		link = (TV_OBJ(TInputLine)*)zend_object_store_get_object(zlink);
	}
	obj = (TV_OBJ(THistory)*)zend_object_store_get_object(getThis());
	obj->obj = new THistory(TRect(ax, ay, bx, by), link ? link->obj : NULL, histid);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TProgressBar, __construct) /* {{{ */
{
	TV_OBJ(TProgressBar) *obj;
	long ax, ay, bx, by, max;

	ax = ay = 1; bx = by = 2;  // just some defaults
	max = 0;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllll", &ax, &ay, &bx, &by, &max);
	obj = (TV_OBJ(TProgressBar)*)zend_object_store_get_object(getThis());
	obj->obj = new TProgressBar(TRect(ax, ay, bx, by), max, 0);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TProgressBar, getValue) /* {{{ */
{
	TV_OBJ(TProgressBar) *obj;

	obj = (TV_OBJ(TProgressBar)*)zend_object_store_get_object(getThis());
	RETVAL_LONG(obj->obj->getValue());
}
/* }}} */

PHP_METHOD(TProgressBar, setValue) /* {{{ */
{
	TV_OBJ(TProgressBar) *obj;
	long value;

	value = 0;
	
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &value);
	obj = (TV_OBJ(TProgressBar)*)zend_object_store_get_object(getThis());
	obj->obj->setValue(value);
}
/* }}} */

/*
 *  * Local variables:
 *  * tab-width: 4
 *  * c-basic-offset: 4
 *  * End:
 *  * vim600: noet sw=4 ts=4 fdm=marker
 *  * vim<600: noet sw=4 ts=4
 *  
 */

