/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_dialog.h 2 2005-04-17 13:24:23Z andrey $ */

#ifndef TV_DIALOG_H
#define TV_DIALOG_H

extern "C" {
#include "php.h"
}

#include "tv_common.h"
#include "tv_commands.h"
#define Uses_TDialog
#define Uses_TRect
#define Uses_TInputLine
#define Uses_TButton
#define Uses_TCluster
#define Uses_TCheckBoxes
#define Uses_TRadioButtons
#define Uses_TSItem
#define Uses_TScrollBar
#define Uses_TListBox
#define Uses_TStringCollection
#define Uses_TStaticText
#define Uses_TLabel
#define Uses_THistory
#include <tvision/tv.h>



class TVDialog : public TDialog
{
  public:
	TVDialog(const TRect& bounds, const char *title);
    ~TVDialog();
	void handleEvent(TEvent&);
};

class TVRadioButtons : public TRadioButtons
{
  public:
	TVRadioButtons(const TRect&, TSItem*);
	~TVRadioButtons();
	void movedTo(int);
	void press(int);
};

class TVCheckBoxes : public TCheckBoxes
{
  public:
	TVCheckBoxes(const TRect&, TSItem*);
	~TVCheckBoxes();
	void press(int);
};

class TProgressBar: public TView
{
	static const int colLeft = 0x4f;
	static const int colRight = 0x2f;
	int max;
	int value;
public:
	TProgressBar(const TRect &r, int aMax, int aValue = 0);
	void draw();
	int getValue() { return value; }
	void setValue(int aValue);
};


class TVListBox : public TListBox
{
  public:
    TVListBox(const TRect& bounds, ushort aNumCols, TScrollBar *aScrollBar );
    void focusItemNum(short item);
};

class TNotSortedStrCollection : public TStringCollection
{
  public:
    TNotSortedStrCollection( short aLimit, short aDelta ) : TStringCollection(aLimit, aDelta) {duplicates=True;}
    int compare( void *key1, void *key2 ) {return -1;}
};

class TVStaticText : public TStaticText
{
  public:
    TVStaticText(const TRect& bounds, const char *aText);
    void setText(const char *aText);
};


TV_ALIASED_OBJECT(TDialog, TVDialog)

TV_CE_INIT_D(TDialog);
PHP_METHOD(TDialog, __construct);

TV_OBJECT(TInputLine);
TV_CE_INIT_D(TInputLine);
PHP_METHOD(TInputLine, __construct);
PHP_METHOD(TInputLine, selectAll);
PHP_METHOD(TInputLine, selStart);
PHP_METHOD(TInputLine, selEnd);
PHP_METHOD(TInputLine, getData);

TV_OBJECT(TButton);
TV_CE_INIT_D(TButton);
PHP_METHOD(TButton, __construct);
PHP_METHOD(TButton, press);
PHP_METHOD(TButton, makeDefault);

TV_ALIASED_OBJECT(TCheckBoxes, TVCheckBoxes);
TV_CE_INIT_D(TCheckBoxes);
PHP_METHOD(TCheckBoxes, __construct);
PHP_METHOD(TCheckBoxes, checked);
PHP_METHOD(TCheckBoxes, toggle);

TV_ALIASED_OBJECT(TRadioButtons, TVRadioButtons);
TV_CE_INIT_D(TRadioButtons);
PHP_METHOD(TRadioButtons, __construct);
PHP_METHOD(TRadioButtons, checked);
PHP_METHOD(TRadioButtons, check);

TV_OBJECT(TScrollBar);
TV_CE_INIT_D(TScrollBar);
PHP_METHOD(TScrollBar, __construct);
PHP_METHOD(TScrollBar, setParams);
PHP_METHOD(TScrollBar, setRange);
PHP_METHOD(TScrollBar, setStep);
PHP_METHOD(TScrollBar, setValue);

TV_ALIASED_OBJECT(TListBox, TVListBox);
TV_CE_INIT_D(TListBox);
PHP_METHOD(TListBox, __construct);
PHP_METHOD(TListBox, newList);
PHP_METHOD(TListBox, focusedItem);
PHP_METHOD(TListBox, selectItem);

TV_ALIASED_OBJECT(TStaticText, TVStaticText);
TV_CE_INIT_D(TStaticText);
PHP_METHOD(TStaticText, __construct);
PHP_METHOD(TStaticText, setText);

TV_OBJECT(TLabel);
TV_CE_INIT_D(TLabel);
PHP_METHOD(TLabel, __construct);

TV_OBJECT(THistory);
TV_CE_INIT_D(THistory);
PHP_METHOD(THistory, __construct);

TV_OBJECT(TProgressBar);
TV_CE_INIT_D(TProgressBar);
PHP_METHOD(TProgressBar, __construct);
PHP_METHOD(TProgressBar, getValue);
PHP_METHOD(TProgressBar, setValue);

#endif // TV_DIALOG_H

