/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_editor.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_editor.h"
#include "tv_view.h"
#include "tv_dialog.h"
#include "tv_registry.h"
/* }}} */

#define cpMemo      "\x1A\x1B"

extern TV_CLASS_ENTRY(TView);
extern TV_CLASS_ENTRY(TScrollBar);

TEditr::TEditr( const TRect& bounds,/*{{{*/
                          TScrollBar *aHScrollBar,
                          TScrollBar *aVScrollBar,
                          TIndicator *aIndicator
                        ) :
    TEditor( bounds, aHScrollBar, aVScrollBar, aIndicator, 0 ), readOnly(false)
{
}/*}}}*/

void TEditr::setData(void *rec)/*{{{*/
{
	uint len = strlen((char*)rec);
	setBufSize(len);
	memcpy(&buffer[bufSize - len], rec, len);
	setBufLen(len);
}/*}}}*/

Boolean TEditr::setBufSize( uint newSize )/*{{{*/
{ 
	if( newSize == 0)
        newSize = 0x1000;
    else
        newSize = (newSize + 0x0FFF) & 0xFFFFF000;
    if( newSize != bufSize )
        {
        char *temp = buffer;
        /* Bypass safety pool to allocate buffer, but check for possible
           NULL return value. */
        if( (buffer = (char *) malloc( newSize )) == 0 )
            {
            delete temp;
            return False;
            }
        ulong n = bufLen - curPtr + delCount;
        memcpy( buffer, temp, min( newSize, bufSize ) );
        memmove( &buffer[newSize - n], &temp[bufSize - n], n );
        delete temp;
        bufSize = newSize;
        gapLen = bufSize - bufLen;
        }
    return True;
}/*}}}*/

void TEditr::handleEvent( TEvent& event )/*{{{*/
{
	if (readOnly) {
    TEditor::convertEvent(event);
    if (event.what == evCommand) {
        if (
        event.message.command != cmCopy &&
        event.message.command != cmCharLeft &&
        event.message.command != cmCharRight &&
        event.message.command != cmWordLeft &&
        event.message.command != cmWordRight &&
        event.message.command != cmLineStart &&
        event.message.command != cmLineEnd &&
        event.message.command != cmLineUp &&
        event.message.command != cmLineDown &&
        event.message.command != cmPageUp &&
        event.message.command != cmPageDown &&
        event.message.command != cmTextStart &&
        event.message.command != cmTextEnd &&
        event.message.command != cmStartSelect &&
        event.message.command != cmHideSelect)
        return;
    }
    switch( event.what ) {
		case evKeyDown:
			if( event.keyDown.charScan.charCode == 9 || event.keyDown.charScan.charCode == 13 ||
                ( event.keyDown.charScan.charCode >= 32 && event.keyDown.charScan.charCode < 255 ) ) {
				return;
            }
            break;
	}
	}
	TEditor::handleEvent(event);





}/*}}}*/


TDlgEditor::TDlgEditor( const TRect& bounds,/*{{{*/
                          TScrollBar *aHScrollBar,
                          TScrollBar *aVScrollBar,
                          TIndicator *aIndicator
                        ) :
    TEditr( bounds, aHScrollBar, aVScrollBar, aIndicator)
{
}/*}}}*/

TPalette& TDlgEditor::getPalette() const/*{{{*/
{
    static TPalette palette( cpMemo, sizeof( cpMemo )-1 );
    return palette;
}/*}}}*/

void TDlgEditor::setData(void *rec)/*{{{*/
{
	TEditr::setData(rec);
}/*}}}*/

Boolean TDlgEditor::setBufSize( uint newSize )/*{{{*/
{
	TEditr::setBufSize(newSize);
} /* }}} */

void TDlgEditor::handleEvent( TEvent& event ) /* {{{ */
{
    if( event.what != evKeyDown || event.keyDown.keyCode != kbTab )
        TEditr::handleEvent(event);
} /* }}} */


TV_CLASS_ENTRY(TEditor);

/* TEditor declarations {{{*/
TV_CE_BEGIN(TEditor)
	PHP_ME(TEditor, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TEditor, readOnly,		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TEditor)

TV_OBJECTS_NEW(TEditor)

TV_CE_INIT(TEditor, TV_CE(TView))/*}}}*/

/* TMemo declarations {{{*/
TV_CLASS_ENTRY(TMemo);

TV_CE_BEGIN(TMemo)
	PHP_ME(TMemo, __construct,	NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TMemo)

TV_OBJECTS_NEW(TMemo)

TV_CE_INIT(TMemo, TV_CE(TEditor))/*}}}*/

PHP_METHOD(TEditor, __construct) /* {{{ */
{
	TV_OBJ(TEditor)* obj;
	TV_OBJ(TScrollBar)* vsb;
	TV_OBJ(TScrollBar)* hsb;
	long ax, ay, bx, by;
	zval *zvsb, *zhsb;

	ax = ay = 1; bx = by = 8;
	zvsb = zhsb = NULL;
	vsb = hsb = NULL;

	obj = (TV_OBJ(TEditor)*)zend_object_store_get_object(getThis());

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllO!O!", &ax, &ay, &bx, &by, &zhsb, TV_CE(TScrollBar), &zvsb, TV_CE(TScrollBar));
	
	if (zvsb && !ZVAL_IS_NULL(zvsb)) {
		vsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zvsb);
	}
	if (zhsb && !ZVAL_IS_NULL(zhsb)) {
		hsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zhsb);
	}
	obj->obj = new TEditr(TRect(ax, ay, bx, by), hsb ? hsb->obj : NULL, vsb ? vsb->obj : NULL, NULL);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TEditor, readOnly) /*{{{*/
{
	TV_OBJ(TEditor)* obj;
	zend_bool newval;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &newval) == FAILURE) {
		RETURN_NULL();
	}
	obj=(TV_OBJ(TEditor)*)zend_object_store_get_object(getThis());
	if (ZEND_NUM_ARGS()) {
		obj->obj->setReadOnly(newval);
	}
	if (obj->obj->isReadOnly()) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}/*}}}*/


PHP_METHOD(TMemo, __construct) /* {{{ */
{
	TV_OBJ(TMemo)* obj;
	TV_OBJ(TScrollBar)* vsb;
	TV_OBJ(TScrollBar)* hsb;
	long ax, ay, bx, by;
	zval *zvsb, *zhsb;

	ax = ay = 1; bx = by = 8;
	zvsb = zhsb = NULL;
	vsb = hsb = NULL;

	obj = (TV_OBJ(TMemo)*)zend_object_store_get_object(getThis());

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllO!O!", &ax, &ay, &bx, &by, &zhsb, TV_CE(TScrollBar), &zvsb, TV_CE(TScrollBar));
	
	if (zvsb && !ZVAL_IS_NULL(zvsb)) {
		vsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zvsb);
	}
	if (zhsb && !ZVAL_IS_NULL(zhsb)) {
		hsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zhsb);
	}
	obj->obj = new TDlgEditor(TRect(ax, ay, bx, by), hsb ? hsb->obj : NULL, vsb ? vsb->obj : NULL, NULL);
	addref_and_store(obj);
}
/* }}} */

/*
 *  * Local variables:
 *  * tab-width: 4
 *  * c-basic-offset: 4
 *  * End:
 *  * vim600: noet sw=4 ts=4 fdm=marker
 *  * vim<600: noet sw=4 ts=4
 *  
 */

