/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_menu.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_menu.h"
#include "tv_registry.h"
/* }}} */

/* TMenuItem declarations {{{ */
TV_CLASS_ENTRY(TMenuItem);
TV_CE_BEGIN(TMenuItem)
	PHP_ME(TMenuItem, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TMenuItem, append,      NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, next,        NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, name,        NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, command,     NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, disabled,    NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, keyCode,     NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, helpCtx,     NULL, ZEND_ACC_PUBLIC)
    PHP_ME(TMenuItem, param,       NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TMenuItem, subMenu,     NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TMenuItem)
TV_OBJECTS_NEW(TMenuItem)
TV_CE_INIT(TMenuItem, NULL)
/* }}} */

/* TSubMenu declarations {{{ */
TV_CLASS_ENTRY(TSubMenu);
TV_CE_BEGIN(TSubMenu)
	PHP_ME(TSubMenu, __construct, NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TSubMenu)
TV_OBJECTS_NEW(TSubMenu)
TV_CE_INIT(TSubMenu, TV_CE(TMenuItem))
/* }}} */

/* TMenu declarations {{{ */
TV_CLASS_ENTRY(TMenu);
TV_CE_BEGIN(TMenu)
	PHP_ME(TMenu, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TMenu, items,       NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TMenu, deflt,       NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TMenu)
TV_OBJECTS_NEW(TMenu)
TV_CE_INIT(TMenu, NULL)
/* }}} */

PHP_METHOD(TSubMenu, __construct) /* {{{ */
{
	TV_OBJ(TSubMenu) *obj;
	TV_OBJ(TMenuItem) *mnu;
	char *aName;
	int aNameLen;
	long aKeyCode, aHelpCtx;
	TMenuItem *aNext;
	
	zval *znext;

	aNameLen = aKeyCode = 0;
	aHelpCtx = hcNoContext;
	
	aNext = NULL;
	znext = NULL;
	aName = NULL;

	obj = (TV_OBJ(TSubMenu)*)zend_object_store_get_object(getThis());
    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, 
				"sl|lO!", 
				&aName, &aNameLen,
				&aKeyCode, &aHelpCtx,
				&znext, TV_CE(TMenuItem));
	if (znext && !ZVAL_IS_NULL(znext)) {
		mnu = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(znext);
		aNext = mnu->obj;
	}
	obj->obj = new TSubMenu(aName, aKeyCode, aHelpCtx);
	obj->obj->next = aNext;
}
/* }}} */

PHP_METHOD(TMenuItem, __construct) /* {{{ */
{
	TV_OBJ(TMenuItem) *obj, *mnu;
	TV_OBJ(TMenu) *smnu;
	char *aName, *p;
	int aNameLen, plen;
	long aCommand, aKeyCode, aHelpCtx;
	TMenuItem *aNext;
	TMenu *aSubMenu;
	zval *znext, *zsubmenu;
	int pulldown = -1;

	aNameLen = plen = aCommand = aKeyCode = 0;
	aHelpCtx = hcNoContext;
	aSubMenu = NULL;
	aNext = NULL;
	znext = zsubmenu = NULL;
	aName = p = NULL;

	obj = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(getThis());
    if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, 
				"sll|lsO!", 
				&aName, &aNameLen,
				&aCommand, &aKeyCode, &aHelpCtx,
				&p, &plen,
				&znext, TV_CE(TMenuItem)) != FAILURE) {
		pulldown = 0;
		if (znext && !ZVAL_IS_NULL(znext)) {
			mnu = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(znext);
			aNext = mnu->obj;
		}
	}
    if (pulldown == -1 && zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, 
				"slO!|lO!", 
				&aName, &aNameLen,
				&aKeyCode,
				&zsubmenu, TV_CE(TMenu),
				&aHelpCtx,
				&znext, TV_CE(TMenuItem)) != FAILURE) {
		pulldown = 1;
		if (zsubmenu && !ZVAL_IS_NULL(zsubmenu)) {
			smnu = (TV_OBJ(TMenu)*)zend_object_store_get_object(zsubmenu);
			aSubMenu = smnu->obj;
		}
		if (znext && !ZVAL_IS_NULL(znext)) {
			mnu = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(znext);
			aNext = mnu->obj;
		}
	}
	if (pulldown == -1) {
		php_error(E_ERROR, "Unable to create menu item -- invalid arguments");
		return;
	}
	if (!aNameLen) aName = NULL; // passing an empty string makes no sense unless we want a separator
	if (pulldown) {
		obj->obj = new TMenuItem(aName, aKeyCode, aSubMenu, aHelpCtx, aNext);
	} else {
		obj->obj = new TMenuItem(aName, aName ? aCommand : -1, aKeyCode, aHelpCtx, p, aNext);
	}
	addref_and_store(obj);
}
/* }}} */
PHP_METHOD(TMenuItem, append) /* {{{ */
{
	TV_OBJ(TMenuItem) *obj, *mnu;
	TMenuItem *aNext;
	zval *znext;

	znext = NULL;

	obj = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(getThis());
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &znext, TV_CE(TMenuItem)) == FAILURE) {
		RETURN_NULL();
	}
	mnu = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(znext);
	aNext = mnu->obj;
	obj->obj->append(aNext);
	RETURN_TRUE;
}
/* }}} */
/* TMenuItem properties wrappers {{{ */
TV_LONG_PROP_GETTER_SETTER(TMenuItem, command, 1)
TV_BOOL_PROP_GETTER_SETTER(TMenuItem, disabled, 1)
TV_LONG_PROP_GETTER_SETTER(TMenuItem, keyCode, 1)
TV_LONG_PROP_GETTER_SETTER(TMenuItem, helpCtx, 1)
TV_STRING_PROP_GETTER_SETTER(TMenuItem, name, 0)
TV_STRING_PROP_GETTER_SETTER(TMenuItem, param, 0)
TV_OBJ_PROP_GETTER_SETTER(TMenuItem, TMenuItem, next, 1)
TV_OBJ_PROP_GETTER_SETTER(TMenuItem, TMenu, subMenu, 1)
/* }}} */

PHP_METHOD(TMenu, __construct) /* {{{ */
{
	TV_OBJ(TMenu) *obj;
	TV_OBJ(TMenuItem) *list;
	zval *zlist;
	obj = (TV_OBJ(TMenu)*)zend_object_store_get_object(getThis());
	list = NULL;
	zlist = NULL;
    zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "|O!", &zlist, TV_CE(TMenuItem));
	if (zlist) {
		list = (TV_OBJ(TMenuItem)*)zend_object_store_get_object(zlist);
	}
	if (list) {
		obj->obj = new TMenu(*list->obj);
	} else {
		obj->obj = new TMenu;
	}
	addref_and_store(obj);
}
/* }}} */
/* TMenu properties wrappers {{{ */
TV_OBJ_PROP_GETTER_SETTER(TMenu, TMenuItem, items, 1)
TV_OBJ_PROP_GETTER_SETTER(TMenu, TMenuItem, deflt, 1)
/* }}} */

/*
 * *  * Local variables:
 * *  * tab-width: 4
 * *  * c-basic-offset: 4
 * *  * End:
 * *  * vim600: noet sw=4 ts=4 fdm=marker
 * *  * vim<600: noet sw=4 ts=4
 * *  
*/

