/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_outline.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_outline.h"
#include "tv_registry.h"
#include "tv_view.h"
#include "tv_dialog.h"
/* }}} */

extern TV_CLASS_ENTRY(TView);
extern TV_CLASS_ENTRY(TScrollBar);

#define cpOutline_ "\x06\x1B\x1C\x1D"

TVOutline::TVOutline(TRect bounds, TScrollBar* hsb, TScrollBar* vsb, TNode* root) :/*{{{*/
	TOutline(bounds, hsb, vsb, root)
{
}/*}}}*/

void TVOutline::focused(int i)/*{{{*/
{
	TOutline::focused(i);
	message(owner, evBroadcast, cmListItemSelected, this);
}/*}}}*/

TPalette& TVOutline::getPalette() const/*{{{*/
{
    static TPalette palette( cpOutline_, sizeof( cpOutline_ )-1 );
    return palette;
}/*}}}*/

/* TNode declarations {{{*/
TV_CLASS_ENTRY(TNode);

TV_CE_BEGIN(TNode)
	PHP_ME(TNode, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TNode, text,	 NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TNode)

TV_OBJECTS_NEW(TNode)

TV_CE_INIT(TNode, NULL)/*}}}*/

/* TOutline declarations {{{*/
TV_CLASS_ENTRY(TOutline);

TV_CE_BEGIN(TOutline)
	PHP_ME(TOutline, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, firstThat,   NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, forEach,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, isSelected,  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, expand,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, collapse,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, focused,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, getRoot,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, getChild,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, getNumChildren,	  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, hasChildren,  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, expandAll,   NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TOutline, getNode,	  NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TOutline)

TV_OBJECTS_NEW(TOutline)

TV_CE_INIT(TOutline, TV_CE(TView))/*}}}*/


PHP_METHOD(TNode, __construct) /* {{{ */
{
	TV_OBJ(TNode)* obj;
	char *text;
	int textlen;
	TV_OBJ(TNode) *children, *next;
	zval *zchildren, *znext;
	bool expanded;

	obj = (TV_OBJ(TNode)*)zend_object_store_get_object(getThis());
	zchildren = znext = NULL;
	children = next = NULL;
	expanded = false;

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|O!O!b", &text, &textlen, &zchildren, TV_CE(TNode), &znext, TV_CE(TNode), &expanded);

	if (zchildren && !ZVAL_IS_NULL(zchildren)) {
		children = (TV_OBJ(TNode)*)zend_object_store_get_object(zchildren);
	}

	if (znext && !ZVAL_IS_NULL(znext)) {
		next = (TV_OBJ(TNode)*)zend_object_store_get_object(znext);
	}
	
	obj->obj = new TNode(text, children ? children->obj : NULL, next ? next->obj : NULL, expanded ? True : False);
	addref_and_store(obj);
}
/* }}} */

TV_STRING_PROP_GETTER_SETTER(TNode, text, 0) 

PHP_METHOD(TOutline, __construct) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	long ax, ay, bx, by;
	zval *zhsb, *zvsb, *zroot;
	TV_OBJ(TScrollBar) *vsb, *hsb;
	TV_OBJ(TNode) *root;

	zhsb = zvsb = zroot = NULL;
	vsb = hsb = NULL;

	ax = ay = 1;
	bx = by = 8; 

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllO!O!O!", &ax, &ay, &bx, &by, &zhsb, TV_CE(TScrollBar), &zvsb, TV_CE(TScrollBar), &zroot, TV_CE(TNode));

	if (zvsb && !ZVAL_IS_NULL(zvsb)) {
		vsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zvsb);
	}
	if (zhsb && !ZVAL_IS_NULL(zhsb)) {
		hsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zhsb);
	}
	if (zroot && !ZVAL_IS_NULL(zroot)) {
		root = (TV_OBJ(TNode)*)zend_object_store_get_object(zroot);
	}
	
	obj->obj = new TVOutline(TRect(ax, ay, bx, by), hsb ? hsb->obj : NULL, vsb ? vsb->obj : NULL, root ? root->obj : NULL);
	addref_and_store(obj);
}
/* }}} */

PHP_METHOD(TOutline, expand) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *znode;
	TV_OBJ(TNode) *node;

	znode = NULL;
	node = NULL;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &znode, TV_CE(TNode));

	if (znode && !ZVAL_IS_NULL(znode)) {
		node = (TV_OBJ(TNode)*)zend_object_store_get_object(znode);
	}
	
	if (node) {
		obj->obj->adjust(node->obj, True);
	}
}
/* }}} */

PHP_METHOD(TOutline, collapse) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *znode;
	TV_OBJ(TNode) *node;

	znode = NULL;
	node = NULL;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &znode, TV_CE(TNode));

	if (znode && !ZVAL_IS_NULL(znode)) {
		node = (TV_OBJ(TNode)*)zend_object_store_get_object(znode);
	}
	
	if (node) {
		obj->obj->adjust(node->obj, False);
	}
}
/* }}} */

PHP_METHOD(TOutline, expandAll) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *znode;
	TV_OBJ(TNode) *node;

	znode = NULL;
	node = NULL;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O!", &znode, TV_CE(TNode));

	if (znode && !ZVAL_IS_NULL(znode)) {
		node = (TV_OBJ(TNode)*)zend_object_store_get_object(znode);
	}
	
	if (node) {
		obj->obj->expandAll(node->obj);
	} else {
		obj->obj->expandAll(obj->obj->getRoot());
	}
}
/* }}} */

TNode* walkTree(TOutlineViewer* outLine,/*{{{*/
        zval *callback,
        int& position, Boolean& checkResult, TNode* cur, int level,
        long lines, Boolean lastChild)
{

  Boolean result;
  int j, childCount;
  TNode* ret;
  ushort flags;
  Boolean children;
  zval *retval;
  zval ***call_args;
  TV_OBJ(TNode) *wcur;

  if (cur == 0)
        return 0;

  children = outLine->hasChildren(cur);

  flags = 0;
  if (lastChild)
        flags |= ovLast;

  if (children && outLine->isExpanded(cur))
        flags |=  ovChildren;

  if (! children || outLine->isExpanded(cur))
        flags |= ovExpanded;

  position++;

  ALLOC_CALLBACK_ARGS(call_args, 0, 4);
  *call_args[0] = tv_registry_get(outLine);
  object_init_ex(*call_args[1], TV_CE(TNode));
  wcur = (TV_OBJ(TNode)*)zend_object_store_get_object(*call_args[1]);
  wcur->obj = cur;
  
  ZVAL_LONG(*call_args[2], level);
  ZVAL_LONG(*call_args[3], position);
//  ZVAL_LONG(*call_args[4], lines);
//  ZVAL_LONG(*call_args[5], flags);

  call_user_function_ex(EG(function_table), NULL, callback, &retval, 4, call_args, 0, NULL TSRMLS_CC);
  FREE_CALLBACK_ARGS(call_args, 0, 4);
  result = Boolean(retval->value.lval == 0);
//  result = (*action)(outLine, cur, level, position, lines, flags);
  if (checkResult && result)
        return cur;

  if (children/* && outLine->isExpanded(cur)*/)
  {
    childCount = outLine->getNumChildren(cur);

    if (! lastChild)
        lines |=  1 << level;

    for (j = 0; j < childCount; j++)
    {
       ret = walkTree(outLine, callback, position, checkResult,
               outLine->getChild(cur, j), level + 1, lines,
                       Boolean(j == (childCount - 1)));
       if (ret)
        return ret;
    }
  }
  return 0;
}/*}}}*/

PHP_METHOD(TOutline, getRoot) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());
	TNode *root;


	root = obj->obj->getRoot();
	if (root) {
		TV_OBJ(TNode) *wcur;
		object_init_ex(return_value, TV_CE(TNode));
		wcur = (TV_OBJ(TNode)*)zend_object_store_get_object(return_value);
		wcur->obj = root;
	} else {
		RETVAL_NULL()
	}
}
/* }}} */

PHP_METHOD(TOutline, getChild) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *znode;
	TV_OBJ(TNode) *node;
	long i;
	TNode *child;

	znode = NULL;
	node = NULL;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Ol", &znode, TV_CE(TNode), &i) != SUCCESS) {
		RETURN_NULL();
	}

	node = (TV_OBJ(TNode)*)zend_object_store_get_object(znode);
	
	child = obj->obj->getChild(node->obj, i);
	if (child) {
		TV_OBJ(TNode) *wcur;
		object_init_ex(return_value, TV_CE(TNode));
		wcur = (TV_OBJ(TNode)*)zend_object_store_get_object(return_value);
		wcur->obj = child;
	} else {
		RETVAL_NULL();
	}

}
/* }}} */

PHP_METHOD(TOutline, getNumChildren) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *znode;
	TV_OBJ(TNode) *node;

	znode = NULL;
	node = NULL;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &znode, TV_CE(TNode)) != SUCCESS) {
		RETURN_NULL();
	}

	node = (TV_OBJ(TNode)*)zend_object_store_get_object(znode);
	
	RETVAL_LONG(obj->obj->getNumChildren(node->obj));

}
/* }}} */

PHP_METHOD(TOutline, hasChildren) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *znode;
	TV_OBJ(TNode) *node;

	znode = NULL;
	node = NULL;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &znode, TV_CE(TNode)) != SUCCESS) {
		RETURN_NULL();
	}

	node = (TV_OBJ(TNode)*)zend_object_store_get_object(znode);
	
	RETVAL_BOOL(obj->obj->hasChildren(node->obj) ? true : false);

}
/* }}} */

PHP_METHOD(TOutline, focused) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	TNode *child;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

	RETVAL_LONG(obj->obj->foc);

}
/* }}} */

PHP_METHOD(TOutline, isSelected) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	TNode *child;
	long i;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &i) != SUCCESS) {
		RETURN_NULL();
	}

	RETVAL_BOOL(obj->obj->isSelected(i));

}
/* }}} */

PHP_METHOD(TOutline, getNode) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	TNode *child;
	long i;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &i) != SUCCESS) {
		RETURN_NULL();
	}

	child = obj->obj->getNode(i);
	if (child) {
		TV_OBJ(TNode) *wcur;
		object_init_ex(return_value, TV_CE(TNode));
		wcur = (TV_OBJ(TNode)*)zend_object_store_get_object(return_value);
		wcur->obj = child;
	} else {
		RETVAL_NULL();
	}

}
/* }}} */

PHP_METHOD(TOutline, firstThat) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *callback;
	TNode *found;
	int position;
	Boolean checkResult;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &callback, TV_CE(TNode));
	if (!zend_is_callable(callback, false, NULL)) {
			RETURN_NULL();
	}

	position = -1;
	checkResult = True;
	found = walkTree(obj->obj, callback, position, checkResult, obj->obj->getRoot(), 0, 0, True);
	if (!found) {
		RETVAL_NULL();
	} else {
	  TV_OBJ(TNode) *wcur;
	  object_init_ex(return_value, TV_CE(TNode));
	  wcur = (TV_OBJ(TNode)*)zend_object_store_get_object(return_value);
	  wcur->obj = found;
	}
}
/* }}} */

PHP_METHOD(TOutline, forEach) /* {{{ */
{
	TV_OBJ(TOutline)* obj;
	zval *callback;
	TNode *found;
	int position;
	Boolean checkResult;

	obj = (TV_OBJ(TOutline)*)zend_object_store_get_object(getThis());

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &callback, TV_CE(TNode));
	if (!zend_is_callable(callback, false, NULL)) {
			RETURN_NULL();
	}

	position = -1;
	checkResult = False;
	walkTree(obj->obj, callback, position, checkResult, obj->obj->getRoot(), 0, 0, True);
}
/* }}} */


/*
 * *  * Local variables:
 * *  * tab-width: 4
 * *  * c-basic-offset: 4
 * *  * End:
 * *  * vim600: noet sw=4 ts=4 fdm=marker
 * *  * vim<600: noet sw=4 ts=4
 * *  
*/

