/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_rect.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_rect.h"
#include "tv_registry.h"
/* }}} */

/* TRect declarations {{{*/
TV_CLASS_ENTRY(TRect);

TV_CE_BEGIN(TRect)
	PHP_ME(TRect, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, move,        NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, grow,        NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, intersect,   NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, union,       NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, contains,    NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, isEmpty,     NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, a,           NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TRect, b,           NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TRect)

TV_OBJECTS_NEW(TRect)

TV_CE_INIT(TRect, NULL)/*}}}*/

PHP_METHOD(TRect, __construct) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	long ax, ay, bx, by;
	obj = (TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
	TRect *rect;

    zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llll", &ax, &ay, &bx, &by);
	rect = new TRect(ax, ay, bx, by);
	obj->obj = rect;
	addref_and_store(obj);
}
/* }}} */
PHP_METHOD(TRect, move) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	long dx, dy;
	dx = dy = 0;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &dx, &dy) != FAILURE) {
		obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
		obj->obj->move(dx, dy);
	}

}
/* }}} */
PHP_METHOD(TRect, grow) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	long dx, dy;
	dx = dy = 0;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &dx, &dy) != FAILURE) {
		obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
		obj->obj->grow(dx, dy);
	}

}
/* }}} */
PHP_METHOD(TRect, intersect) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	TV_OBJ(TRect)* ro;
	zval *r;
	long dx, dy;
	dx = dy = 0;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &r, TV_CE(TRect)) != FAILURE) {
		obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
		ro=(TV_OBJ(TRect)*)zend_object_store_get_object(r);
		obj->obj->intersect(*ro->obj);
	}

}
/* }}} */
PHP_METHOD(TRect, union) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	TV_OBJ(TRect)* ro;
	zval *r;
	long dx, dy;
	dx = dy = 0;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &r, TV_CE(TRect)) != FAILURE) {
		obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
		ro=(TV_OBJ(TRect)*)zend_object_store_get_object(r);
		obj->obj->Union(*ro->obj);
	}

}
/* }}} */
PHP_METHOD(TRect, contains) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	long x, y;
	x = y = -1;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &x, &y) != FAILURE) {
		obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
		TPoint p;
		p.x = x; p.y = y;
		RETURN_BOOL(obj->obj->contains(p));
	}
	RETURN_FALSE;
}
/* }}} */
PHP_METHOD(TRect, isEmpty) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
	RETURN_BOOL(obj->obj->isEmpty());
}
/* }}} */
PHP_METHOD(TRect, a) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	zval *i;
	obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
	array_init (return_value);
	MAKE_STD_ZVAL(i);
	ZVAL_LONG(i, obj->obj->a.x);
	add_assoc_zval_ex(return_value, "x", 2, i);
	MAKE_STD_ZVAL(i);
	ZVAL_LONG(i, obj->obj->a.y);
	add_assoc_zval_ex(return_value, "y", 2, i);
}
/* }}} */
PHP_METHOD(TRect, b) /* {{{ */
{
	TV_OBJ(TRect)* obj;
	zval *i;
	obj=(TV_OBJ(TRect)*)zend_object_store_get_object(getThis());
	array_init (return_value);
	MAKE_STD_ZVAL(i);
	ZVAL_LONG(i, obj->obj->b.x);
	add_assoc_zval_ex(return_value, "x", 2, i);
	MAKE_STD_ZVAL(i);
	ZVAL_LONG(i, obj->obj->b.y);
	add_assoc_zval_ex(return_value, "y", 2, i);
}
/* }}} */

/*
 * *  * Local variables:
 * *  * tab-width: 4
 * *  * c-basic-offset: 4
 * *  * End:
 * *  * vim600: noet sw=4 ts=4 fdm=marker
 * *  * vim<600: noet sw=4 ts=4
 * *  
*/

