/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_table.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
}
#include "tv_table.h"
#include "tv_dialog.h"
#include "tv_registry.h"
#include "tv_view.h"
#include "tv_event.h"
/* }}} */

#define separatorChar '\xB3'
#define cpTable "\x19\x1A\x1B\x1C\x1D"
// #define cpTable "\x03\x03\x04\x05\x06"

extern TV_CLASS_ENTRY(TView);
extern TV_CLASS_ENTRY(TScrollBar);
extern TV_CLASS_ENTRY(TEvent);

TTable::TTable(zval *_thisptr, const TRect& bounds, TScrollBar *aHScrollBar, TScrollBar *aVScrollBar) :/* {{{ */
	TView(bounds), thisptr(_thisptr), hsb(aHScrollBar), vsb(aVScrollBar), topItem(0), selected(0), xOffset(0)
{
    options |= ofFirstClick | ofSelectable;
    eventMask |= evBroadcast;
	selectItem(0);
} /* }}} */

void TTable::selectItem(int n)/*{{{*/
{
	int nRows = getRowCount();
	if (n < 0) {
		n = 0;
	}
	if (n >= nRows) {
		n = nRows - 1;
	}
	if (n != selected) {
		selected = n;
		message(owner, evBroadcast, cmListItemSelected, this);
	} else {
		selected = n;
	}
}/*}}}*/

TV_STD_HANDLEEVENT_BEGIN(TTable)/*{{{*/
	TPoint mouse;
	TView::handleEvent(event);
	rowCount = -1;
	if (event.what == evMouseDown) {
		mouse = makeLocal(event.mouse.where);
		if (mouseInView(event.mouse.where) && mouse.y > 0) {
			selectItem(mouse.y - 1 + topItem);
			drawView();
			clearEvent(event);
		}
	} else if (event.what == evKeyDown) {
	    switch (event.keyDown.keyCode)
        {
			case kbUp:
				selectItem(selected-1);
				drawView();
				clearEvent(event);
                break;
            case kbDown:
				selectItem(selected+1);
				drawView();
				clearEvent(event);
                break;
            case kbPgDn:
				selectItem(selected + size.y);
				drawView();
				clearEvent(event);
				break;
            case kbPgUp:
				selectItem(selected - size.y);
				drawView();
				clearEvent(event);
				break;
            case kbHome:
				selectItem(0);
				drawView();
				clearEvent(event);
				break;
            case kbEnd:
				selectItem(getRowCount()-1);
				drawView();
				clearEvent(event);
				break;
		}
	} else if ( event.what == evBroadcast ) {
		if (event.message.command == cmScrollBarClicked) {
			if( vsb == event.message.infoPtr ) {
   				selectItem(vsb->value);
                drawView();
			}
		}
	}
TV_STD_HANDLEEVENT_BODY
TV_STD_HANDLEEVENT_END/*}}}*/

void TTable::setState( ushort aState, Boolean enable )/*{{{*/
{
    TView::setState( aState, enable );
    if( (aState & (sfSelected | sfActive | sfVisible)) != 0 ) {
        drawView();
    }
	if (aState & sfVisible != 0 && vsb) {
		vsb->setState(sfVisible, enable);
	}
}/*}}}*/

void TTable::draw()/*{{{*/
{
	TDrawBuffer b;
	int widths[80];
	int i, j;
	int nCols = getColumnCount();
	if (nCols > 80) {
		nCols = 80;
	}
	rowCount = -1;
	if (vsb) {
		if (getRowCount() > size.y - 1) {
			vsb->setParams(selected, 0, getRowCount() - 1, size.y, 1);
			vsb->show();
			vsb->drawView();
		} else {
			vsb->hide();
		}
	}
	if (topItem > selected) {
		topItem = selected;
	} else 	if (selected - topItem >= size.y - 1) {
		topItem = selected - size.y +2;
	}

	for (i = 0; i < nCols; i++) {
		widths[i] = getColumnWidth(i);
		if (widths[i] > 80) widths[i] = 80;
	}
	int width = 0;
	b.moveChar( 0, ' ', getColor(1), size.x );
	for (j = 0; j < nCols; j++) {
		char buf[81];
		if (j) {
			b.moveChar(width, separatorChar, getColor(1), 1);
			width++;
		}
		getColumnTitle(j, buf, widths[j]);
		b.moveStr(width, buf, getColor(1));
		width += widths[j];
	}
	writeLine( 0, 0, size.x, 1, b );
	for (i = 1; i < size.y; i++) {
		width = 0;

				if (i - 1 + topItem == selected) {
					if( (state&sfSelected) == sfSelected ) {
						b.moveChar( 0, ' ', getColor(3), size.x );
					} else {
						b.moveChar( 0, ' ', getColor(4), size.x );
					}
				} else {
					b.moveChar( 0, ' ', getColor(2), size.x );
				}

		for (j = 0; j < nCols; j++) {
			char buf[81];
			if (j) {
				b.moveChar(width, separatorChar, getColor(5), 1);
				width++;
			}
			if (i - 1 + topItem < getRowCount()) {
				getText(i - 1 + topItem, j, buf, widths[j]);
				if (i - 1 + topItem == selected) {
					if( (state&sfSelected) == sfSelected ) {
						b.moveStr(width, buf, getColor(3));
					} else {
						b.moveStr(width, buf, getColor(4));
					}
				} else {
					b.moveStr(width, buf, getColor(2));
				}
			}
			width += widths[j];
		}
		writeLine( 0, i, size.x, 1, b );
	}
}/*}}}*/

void TTable::getText(int row, int col, char* buf, int maxlen)/*{{{*/
{
	zval method_name, *retval;
	zval ***call_args;
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "getText", strlen("getText"), 1);
	ALLOC_CALLBACK_ARGS(call_args, 0, 2);
	ZVAL_LONG(*call_args[0], row)
	ZVAL_LONG(*call_args[1], col)
	if (call_user_function_ex(NULL, &thisptr, &method_name, &retval, 2, call_args, 1, NULL TSRMLS_CC) != FAILURE) {
		SEPARATE_ZVAL(&retval);
		convert_to_string(retval);
		if (Z_STRLEN_P(retval) < maxlen) {
			memcpy(buf, Z_STRVAL_P(retval), Z_STRLEN_P(retval));
			buf[Z_STRLEN_P(retval)] = 0;
		} else {
			memcpy(buf, Z_STRVAL_P(retval), maxlen);
			buf[maxlen] = 0;
		}
	} else {
		buf[0] = 0;
	}
	FREE_CALLBACK_ARGS(call_args, 0, 2);
}/*}}}*/

void TTable::getColumnTitle(int col, char* buf, int maxlen)/*{{{*/
{
	zval method_name, *retval;
	zval ***call_args;
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "getColumnTitle", strlen("getColumnTitle"), 1);
	ALLOC_CALLBACK_ARGS(call_args, 0, 1);
	ZVAL_LONG(*call_args[0], col);
	if (call_user_function_ex(NULL, &thisptr, &method_name, &retval, 1, call_args, 1, NULL TSRMLS_CC) != FAILURE) {
		SEPARATE_ZVAL(&retval);
		convert_to_string(retval);
		if (Z_STRLEN_P(retval) < maxlen) {
			memcpy(buf, Z_STRVAL_P(retval), Z_STRLEN_P(retval));
			buf[Z_STRLEN_P(retval)] = 0;
		} else {
			memcpy(buf, Z_STRVAL_P(retval), maxlen);
			buf[maxlen] = 0;
		}
	} else {
		buf[0] = 0;
	}
	FREE_CALLBACK_ARGS(call_args, 0, 1);
}/*}}}*/

int TTable::getRowCount()/*{{{*/
{
	if (rowCount >=0 ) {
		return rowCount;
	}
	zval method_name, *retval;
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "getRowCount", 11, 1);
	if (call_user_function_ex(NULL, &thisptr, &method_name, &retval, 0, NULL, 0, NULL TSRMLS_CC) != FAILURE) {
		SEPARATE_ZVAL(&retval);
		convert_to_long(retval);
		rowCount = Z_LVAL_P(retval);
	} else {
		rowCount = 0;
	}
	return rowCount;
}/*}}}*/

int TTable::getColumnCount()/*{{{*/
{
	zval method_name, *retval;
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "getColumnCount", strlen("getColumnCount"), 1);
	if (call_user_function_ex(NULL, &thisptr, &method_name, &retval, 0, NULL, 0, NULL TSRMLS_CC) != FAILURE) {
		convert_to_long(retval);
		return Z_LVAL_P(retval);
	} else {
		return 0;
	}
}/*}}}*/

int TTable::getColumnWidth(int n)/*{{{*/
{
	zval method_name, retval, *pretval;
	zval ***call_args;
	INIT_ZVAL(retval);
	pretval = &retval;
	INIT_PZVAL(&method_name);
	ZVAL_STRINGL(&method_name, "getColumnWidth", strlen("getColumnWidth"), 1);
	ALLOC_CALLBACK_ARGS(call_args, 0, 1);
	ZVAL_LONG(*call_args[0], n);
	if (call_user_function_ex(NULL, &thisptr, &method_name, &pretval, 1, call_args, 1, NULL TSRMLS_CC) != FAILURE) {
		FREE_CALLBACK_ARGS(call_args, 0, 1);
		convert_to_long(pretval);
		return Z_LVAL_P(pretval);
	} else {
		FREE_CALLBACK_ARGS(call_args, 0, 1);
		return 0;
	}
}/*}}}*/

TPalette& TTable::getPalette() const/*{{{*/
{
    static TPalette palette( cpTable, sizeof( cpTable )-1 );
    return palette;
}/*}}}*/

int TTable::getSelectedRow() /*{{{*/
{
    return selected;
}/*}}}*/

TTable::~TTable()/*{{{*/
{
}/*}}}*/

TV_CLASS_ENTRY(TTable);

TV_CE_BEGIN(TTable)/*{{{*/
	PHP_ME(TTable, __construct,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TTable, getRowCount,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TTable, getColumnCount,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TTable, getColumnTitle,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TTable, getText,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TTable, getSelectedRow,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TTable, selectRow,		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TTable)/*}}}*/

TV_OBJECTS_NEW(TTable)

TV_CE_INIT(TTable, TV_CE(TView))

PHP_METHOD(TTable, __construct) /* {{{ */
{
	TV_OBJ(TTable)* obj;
	TV_OBJ(TScrollBar)* vsb;
	TV_OBJ(TScrollBar)* hsb;
	long ax, ay, bx, by;
	zval *zvsb, *zhsb;

	ax = ay = 1; bx = by = 8;
	zvsb = zhsb = NULL;
	vsb = hsb = NULL;

	obj = (TV_OBJ(TTable)*)zend_object_store_get_object(getThis());

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llllO!O!", &ax, &ay, &bx, &by, &zhsb, TV_CE(TScrollBar), &zvsb, TV_CE(TScrollBar));
	
	if (zvsb && !ZVAL_IS_NULL(zvsb)) {
		vsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zvsb);
	}
	if (zhsb && !ZVAL_IS_NULL(zhsb)) {
		hsb = (TV_OBJ(TScrollBar)*)zend_object_store_get_object(zhsb);
	}
	obj->obj = new TTable(getThis(), TRect(ax, ay, bx, by), hsb ? hsb->obj : NULL, vsb ? vsb->obj : NULL);
	addref_and_store(obj);
}  /* }}} */

PHP_METHOD(TTable, getColumnTitle) /* {{{ */
{
	RETVAL_STRING("", 1);
} /* }}} */

PHP_METHOD(TTable, getColumnCount) /* {{{ */
{
	RETVAL_LONG(0);
} /* }}} */

PHP_METHOD(TTable, getRowCount) /* {{{ */
{
	RETVAL_LONG(0);
} /* }}} */

PHP_METHOD(TTable, getText) /* {{{ */
{
	RETVAL_STRING("", 1);
} /* }}} */

PHP_METHOD(TTable, getColumnWidth) /* {{{ */
{
	RETVAL_LONG(0);
} /* }}} */

PHP_METHOD(TTable, getSelectedRow) /* {{{ */
{
	TV_OBJ(TTable)* obj;
	obj = (TV_OBJ(TTable)*)zend_object_store_get_object(getThis());
	RETVAL_LONG(obj->obj->getSelectedRow());
} /* }}} */

PHP_METHOD(TTable, selectRow) /* {{{ */
{
	long row;
    bool force = false;
	TV_OBJ(TTable)* obj;
	obj = (TV_OBJ(TTable)*)zend_object_store_get_object(getThis());

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|b", &row, &force) == FAILURE) {
		RETURN_FALSE;
	}
	obj->obj->selectItem(row);
    if (force) {
        message(obj->obj->owner, evBroadcast, cmListItemSelected, obj->obj);
    }
	RETURN_TRUE;
} /* }}} */

/*
 *  * Local variables:
 *  * tab-width: 4
 *  * c-basic-offset: 4
 *  * End:
 *  * vim600: noet sw=4 ts=4 fdm=marker
 *  * vim<600: noet sw=4 ts=4
 *  
 */

