/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2005 Andrey Demenev                                    |
  +----------------------------------------------------------------------+
  | This source file is subject to the BSD license, that is bundled with |
  | this package in the file LICENSE                                     |
  +----------------------------------------------------------------------+
  | Author:  Andrey Demenev <demenev@gmail.com>                          |
  +----------------------------------------------------------------------+
*/

/* $Id: tv_view.cpp 2 2005-04-17 13:24:23Z andrey $ */

/* includes {{{ */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "zend_exceptions.h"
#include "ext/standard/info.h"
}
#include "tv_view.h"
#include "tv_event.h"
#include "tv_registry.h"
/* }}} */

#define cpBlueWindow_ "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f"\
    "\x50\x51\x52\x53\x54\x55\x56\x57\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f"

extern TV_CLASS_ENTRY(TEvent);

TVWindow::TVWindow(const TRect& bounds, const char *title, short number) :/*{{{*/
	TWindow(bounds, title, number), TWindowInit( &TVWindow::initFrame )
{
}/*}}}*/

TPalette& TVWindow::getPalette() const/*{{{*/
{
	static TPalette palette( cpBlueWindow_, strlen(cpBlueWindow_) );
	return palette;
}/*}}}*/

TV_STD_HANDLEEVENT_BEGIN(TVWindow)/*{{{*/
TWindow::handleEvent(event);
TV_STD_HANDLEEVENT_BODY
TV_STD_HANDLEEVENT_END/*}}}*/

/* TView declarations {{{ */
TV_CLASS_ENTRY(TView);
TV_CE_BEGIN(TView)
	PHP_ME(TView, growTo,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, moveTo,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, hide,				NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, show,				NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, focus,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, hideCursor,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, showCursor,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, dataSize,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, getData,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, setData,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, setCommandState,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, commandEnabled,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, execute,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, endModal,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, select,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, makeFirst,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, drawView,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, options,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, growMode,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, clearEvent,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, nextView,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, getExtent,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, getBounds,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, message,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TView, __free,			NULL, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
TV_CE_END(TView)
TV_OBJECTS_NEW(TView)
TV_CE_INIT(TView, NULL)
/* }}} */

/* TGroup declarations {{{ */
TV_CLASS_ENTRY(TGroup);
TV_CE_BEGIN(TGroup)
	PHP_ME(TGroup, execView,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TGroup, remove,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TGroup, selectNext,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TGroup, focusNext,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TGroup, insert,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TGroup, firstView, 		NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TGroup)
TV_OBJECTS_NEW(TGroup)
TV_CE_INIT(TGroup, TV_CE(TView))
/* }}} */

/* TDeskTop declarations {{{ */
TV_CLASS_ENTRY(TDeskTop);
TV_CE_BEGIN(TDeskTop)
	PHP_ME(TDeskTop, __construct,		NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TDeskTop, cascade,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TDeskTop, tile,				NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TDeskTop)
TV_OBJECTS_NEW(TDeskTop)
TV_CE_INIT(TDeskTop, TV_CE(TGroup))
/* }}} */

/* TWindow declarations {{{ */
TV_CLASS_ENTRY(TWindow);
TV_CE_BEGIN(TWindow)
	PHP_ME(TWindow, __construct,	NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TWindow, close,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TWindow, title,			NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TWindow, zoom,			NULL, ZEND_ACC_PUBLIC)
TV_CE_END(TWindow)
TV_OBJECTS_NEW(TWindow)
TV_CE_INIT(TWindow, TV_CE(TGroup))
/* }}} */


PHP_METHOD(TView, __free) /*{{{*/
{
	TV_OBJ(TView)* app;
	app=(TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	TObject::destroy(app->obj);
//	delete app->obj;
    if (app->ref) {
        app->ref = 0;
        ZVAL_DELREF(getThis());
    }
	tv_registry_del(app->obj);
}/*}}}*/

PHP_METHOD(TView, growTo) /* {{{ */
{
	TV_OBJ(TView) *obj;
	long x, y;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &x, &y) == FAILURE) {
		RETURN_FALSE;
	}
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->growTo(x,y);
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, moveTo) /* {{{ */
{
	TV_OBJ(TView) *obj;
	long x, y;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &x, &y) == FAILURE) {
		RETURN_FALSE;
	}
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->moveTo(x,y);
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, hide) /* {{{ */
{
	TV_OBJ(TView) *obj;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->hide();
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, show) /* {{{ */
{
	TV_OBJ(TView) *obj;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->show();
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, hideCursor) /* {{{ */
{
	TV_OBJ(TView) *obj;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->hideCursor();
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, showCursor) /* {{{ */
{
	TV_OBJ(TView) *obj;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->showCursor();
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, focus) /* {{{ */
{
	TV_OBJ(TView) *obj;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	if (obj->obj->focus()) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}
/* }}} */

PHP_METHOD(TView, dataSize) /* {{{ */
{
	TV_OBJ(TView) *obj;
	long datasize;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	datasize = obj->obj->dataSize();
	RETVAL_LONG(datasize);
}
/* }}} */

PHP_METHOD(TView, getData) /* {{{ */
{
	TV_OBJ(TView) *obj;
	long datasize;
	char *data;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	datasize = obj->obj->dataSize();
	data = (char*)emalloc(datasize + 1);
	data[datasize] = 0;
	obj->obj->getData(data);
	RETVAL_STRINGL(data, datasize, 0);
}
/* }}} */

PHP_METHOD(TView, setData) /* {{{ */
{
	TV_OBJ(TView) *obj;
	long datasize;
	char *data;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &data, &datasize) == FAILURE) {
		RETURN_FALSE;
	}
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	obj->obj->setData(data);
	RETURN_TRUE;
}
/* }}} */

PHP_METHOD(TView, setCommandState) /* {{{ */
{
	long cmd;
	ushort command;
	zend_bool state;
	TV_OBJ(TView)* app;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lb", &cmd, &state) == FAILURE) {
		RETURN_FALSE;
	}
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	command = cmd;
	if (state) {
		app->obj->enableCommand(command);
	} else {
		app->obj->disableCommand(command);
	}
}
/* }}} */

PHP_METHOD(TView, commandEnabled) /* {{{ */
{
	long cmd;
	TV_OBJ(TView)* app;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &cmd) == FAILURE) {
		RETURN_FALSE;
	}
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	if (app->obj->commandEnabled(cmd)) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}
/* }}} */

PHP_METHOD(TView, endModal) /* {{{ */
{
	long cmd;
	TV_OBJ(TView)* app;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &cmd) == FAILURE) {
		RETURN_FALSE;
	}
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	app->obj->endModal(cmd);
}
/* }}} */

PHP_METHOD(TView, execute) /* {{{ */
{
	long ret;
	TV_OBJ(TView)* app;
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	ret = app->obj->execute();
	RETVAL_LONG(ret);
}
/* }}} */

PHP_METHOD(TView, select) /* {{{ */
{
	TV_OBJ(TView)* app;
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	app->obj->select();
}
/* }}} */

PHP_METHOD(TView, drawView) /* {{{ */
{
	TV_OBJ(TView)* app;
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	app->obj->drawView();
}
/* }}} */

PHP_METHOD(TView, makeFirst) /* {{{ */
{
	TV_OBJ(TView)* app;
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	app->obj->makeFirst();
}
/* }}} */

PHP_METHOD(TView, options) /* {{{ */
{
	TV_OBJ(TView)* app;
	long opts;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &opts);
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	if (ZEND_NUM_ARGS()) {
		app->obj->options = opts;
	}
	RETVAL_LONG(app->obj->options);
}
/* }}} */

PHP_METHOD(TView, growMode) /* {{{ */
{
	TV_OBJ(TView)* app;
	long opts;
	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &opts);
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	if (ZEND_NUM_ARGS()) {
		app->obj->growMode = opts;
	}
	RETVAL_LONG(app->obj->growMode);
}
/* }}} */

PHP_METHOD(TView, clearEvent)/* {{{ */
{
	TV_OBJ(TView)* app;
	zval *zevent;
	TV_OBJ(TEvent) *event;
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &zevent, TV_CE(TEvent)) == FAILURE) {
        RETURN_FALSE;
    }
	app = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	event = (TV_OBJ(TEvent)*)zend_object_store_get_object(zevent);
	event->obj->what = 0;
	event->obj->message.infoPtr = app->obj;
	RETURN_TRUE;
} /* }}} */

PHP_METHOD(TView, nextView) /* {{{ */
{
	TV_OBJ(TView) *obj;
    TView *next;
	zval *zobj;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	next = obj->obj->nextView();
	while (next) {
		zobj = tv_registry_get(next);
        if (zobj) {
            RETVAL_ZVAL(zobj, 1, 0);
            return;
        }
        next = next->nextView();
	}
	RETVAL_NULL();
}
/* }}} */

PHP_METHOD(TView, getExtent) /* {{{ */
{
	TRect extent;
	TV_OBJ(TView) *obj;
	zval *zobj, *point;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	extent = obj->obj->getExtent();
	array_init(return_value);
	MAKE_STD_ZVAL(point);
	array_init(point);
	add_assoc_long(point, "x",   extent.a.x);
	add_assoc_long(point, "y",   extent.a.y);
	zend_hash_update(Z_ARRVAL_P(return_value), "a", 2, &point, sizeof(zval *), NULL);
	MAKE_STD_ZVAL(point);
	array_init(point);
	add_assoc_long(point, "x",   extent.b.x);
	add_assoc_long(point, "y",   extent.b.y);
	zend_hash_update(Z_ARRVAL_P(return_value), "b", 2, &point, sizeof(zval *), NULL);
}
/* }}} */

PHP_METHOD(TView, getBounds) /* {{{ */
{
	TRect extent;
	TV_OBJ(TView) *obj;
	zval *zobj, *point;
	obj = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	extent = obj->obj->getBounds();
	array_init(return_value);
	MAKE_STD_ZVAL(point);
	array_init(point);
	add_assoc_long(point, "x",   extent.a.x);
	add_assoc_long(point, "y",   extent.a.y);
	zend_hash_update(Z_ARRVAL_P(return_value), "a", 2, &point, sizeof(zval *), NULL);
	MAKE_STD_ZVAL(point);
	array_init(point);
	add_assoc_long(point, "x",   extent.b.x);
	add_assoc_long(point, "y",   extent.b.y);
	zend_hash_update(Z_ARRVAL_P(return_value), "b", 2, &point, sizeof(zval *), NULL);
}
/* }}} */

PHP_METHOD(TView, message) /* {{{ */
{
	zval *zview, *zwho, *zinfoPtr;
	TV_OBJ(TView)* view;
	TV_OBJ(TView)* who;
	TV_OBJ(TView)* infoPtr;
	long event, command;

	zinfoPtr = NULL;
	infoPtr = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Oll|O!", &zwho, TV_CE(TView), &event, &command, &zinfoPtr, TV_CE(TView))) {
		RETURN_FALSE;
	}
	view = (TV_OBJ(TView)*)zend_object_store_get_object(getThis());
	who = (TV_OBJ(TView)*)zend_object_store_get_object(zwho);
	if (zinfoPtr) {
		infoPtr = (TV_OBJ(TView)*)zend_object_store_get_object(zinfoPtr);
	}
	message(who->obj, event, command, infoPtr ? infoPtr->obj : NULL);
}
/* }}} */


PHP_METHOD(TGroup, execView) /* {{{ */
{
	zval *zview;
	TV_OBJ(TView)* view;
	TV_OBJ(TGroup)* grp;
	long ret;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &zview, TV_CE(TView)) == FAILURE) {
		RETURN_FALSE;
	}
	grp = (TV_OBJ(TGroup)*)zend_object_store_get_object(getThis());
	view = (TV_OBJ(TView)*)zend_object_store_get_object(zview);
	ret = grp->obj->execView(view->obj);
	RETVAL_LONG(ret);
}
/* }}} */

PHP_METHOD(TGroup, remove) /* {{{ */
{
	zval *zview;
	TV_OBJ(TView)* view;
	TV_OBJ(TGroup)* grp;
	long ret;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &zview, TV_CE(TView)) == FAILURE) {
		RETURN_FALSE;
	}
	grp = (TV_OBJ(TGroup)*)zend_object_store_get_object(getThis());
	view = (TV_OBJ(TView)*)zend_object_store_get_object(zview);
	grp->obj->remove(view->obj);
}
/* }}} */

PHP_METHOD(TGroup, insert) /* {{{ */
{
	zval *zview;
	TV_OBJ(TView)* view;
	TV_OBJ(TGroup)* grp;
	long ret;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &zview, TV_CE(TView)) == FAILURE) {
		RETURN_FALSE;
	}
	grp = (TV_OBJ(TGroup)*)zend_object_store_get_object(getThis());
	view = (TV_OBJ(TView)*)zend_object_store_get_object(zview);
	grp->obj->insert(view->obj);
}
/* }}} */

PHP_METHOD(TGroup, selectNext) /* {{{ */
{
	zend_bool forward;
	TV_OBJ(TGroup)* grp;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &forward) == FAILURE) {
		RETURN_FALSE;
	}
	grp = (TV_OBJ(TGroup)*)zend_object_store_get_object(getThis());
	grp->obj->selectNext(forward ? True : False);
}
/* }}} */

PHP_METHOD(TGroup, focusNext) /* {{{ */
{
	zend_bool forward;
	TV_OBJ(TGroup)* grp;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &forward) == FAILURE) {
		RETURN_FALSE;
	}
	grp = (TV_OBJ(TGroup)*)zend_object_store_get_object(getThis());
	grp->obj->focusNext(forward ? True : False);
}
/* }}} */

PHP_METHOD(TGroup, firstView) /* {{{ */
{
	TV_OBJ(TGroup)* wnd;
    TView *first;
    zval *zobj;

	wnd = (TV_OBJ(TGroup)*)zend_object_store_get_object(getThis());
    
	first = wnd->obj->first();
    if (first) {
		zobj = tv_registry_get(first);
        if (zobj) {
            RETVAL_ZVAL(zobj, 1, 0);
            return;
        }
    }
    RETVAL_NULL();
}
/* }}} */


PHP_METHOD(TWindow, __construct) /* {{{ */
{
	long ax, ay, bx, by;
	char *title;
	int title_len;
	long number;
	TV_OBJ(TWindow)* wnd;

	number = 0;
	title = "";
	ax = ay = 1;
	bx = by = 8;

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lllls|l", &ax, &ay, &bx, &by, &title, &title_len, &number);
	wnd = (TV_OBJ(TWindow)*)zend_object_store_get_object(getThis());
	wnd->obj = new TVWindow(TRect(ax, ay, bx, by), title, number);
	addref_and_store(wnd);
}
/* }}} */

PHP_METHOD(TWindow, close) /* {{{ */
{
	TV_OBJ(TWindow)* wnd;
	wnd = (TV_OBJ(TWindow)*)zend_object_store_get_object(getThis());
	wnd->obj->close();
}
/* }}} */

PHP_METHOD(TWindow, zoom) /* {{{ */
{
	TV_OBJ(TWindow)* wnd;
	wnd = (TV_OBJ(TWindow)*)zend_object_store_get_object(getThis());
	wnd->obj->zoom();
}
/* }}} */

PHP_METHOD(TWindow, title)//{{{
{
	TV_OBJ(TWindow)* obj;
	char *newval;
	int vallen; 
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &newval, &vallen) == FAILURE) {
		RETURN_NULL();
	}
	obj=(TV_OBJ(TWindow)*)zend_object_store_get_object(getThis());
	if (ZEND_NUM_ARGS()) {
		obj->obj->title = newval;
		obj->obj->frame->draw();
	}
	RETVAL_STRING((char*)obj->obj->title, 1);
}//}}}


PHP_METHOD(TDeskTop, __construct) /* {{{ */
{
	long ax, ay, bx, by;
	TV_OBJ(TDeskTop)* wnd;

	ax = ay = 1;
	bx = by = 8;

	zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "llll", &ax, &ay, &bx, &by);
	wnd = (TV_OBJ(TDeskTop)*)zend_object_store_get_object(getThis());
	wnd->obj = new TDeskTop(TRect(ax, ay, bx, by));
	addref_and_store(wnd);
}
/* }}} */

PHP_METHOD(TDeskTop, cascade) /* {{{ */
{
	TV_OBJ(TDeskTop)* wnd;

	wnd = (TV_OBJ(TDeskTop)*)zend_object_store_get_object(getThis());
	wnd->obj->cascade(wnd->obj->getExtent());
}
/* }}} */

PHP_METHOD(TDeskTop, tile) /* {{{ */
{
	TV_OBJ(TDeskTop)* wnd;

	wnd = (TV_OBJ(TDeskTop)*)zend_object_store_get_object(getThis());
	wnd->obj->tile(wnd->obj->getExtent());
}
/* }}} */


/*
 *  * Local variables:
 *  * tab-width: 4
 *  * c-basic-offset: 4
 *  * End:
 *  * vim600: noet sw=4 ts=4 fdm=marker
 *  * vim<600: noet sw=4 ts=4
 *  */

